#!/usr/bin/perl
# $Id: play.pl 358 2016-03-28 18:48:45Z bd $

#
# This script should be rewritten to use linuxtv ioctls directly,
# instead of running "tzap" as a sub-process...
#

use Time::HiRes qw(sleep);

my $PATHPREFIX = "/usr/local/glotv";
my $tzap = "/usr/bin/tzap";
my $conf = "/usr/local/glotv/etc/dvb-channels.conf";
my $dvr_device = "/dev/dvb/adapter0/dvr0";
my $play_pid = $$;
my $tzap_pid;
my $restart;
my $mpegbufsize = 32768;

sub cleanup_and_exit {
    kill 'SIGTERM', $tzap_pid if $tzap_pid;
    unlink "$PATHPREFIX/var/dvb-playing-channel";
    unlink "$PATHPREFIX/var/dvb-play.pid";
    system "pkill tzap";  # Temporary workaround for strange tzap issue...
    exit(0);
}

sub terminate_tzap_fork {
    kill 'SIGTERM', $tzap_pid;
    exit(0);
}

sub get_requested_channel {
    open CHAN, '<', "$PATHPREFIX/var/dvb-requested-channel"
	or die "Couldn't read $PATHPREFIX/var/dvb-requested-channel: $!\n";
    my $chan = <CHAN>;
    close CHAN;
    chomp $chan;
    return $chan;
}

$SIG{PIPE} = 'IGNORE';
$SIG{CHLD} = 'IGNORE';

if (open PID, '<', "$PATHPREFIX/var/dvb-play.pid") {
    my $pid = <PID>;
    close PID;
    chomp $pid;
    if ($pid) {
	kill 'SIGTERM', $pid;
	sleep 0.1;
    }
}
open PID, '>', "$PATHPREFIX/var/dvb-play.pid"
    or die "Couldn't write to $PATHPREFIX/var/dvb-play.pid: $!";
print PID $play_pid,"\n";
close PID;

while (1) {
    my $chan = &get_requested_channel();
    $tzap_pid = open TZAP, '-|', "$tzap -c $conf -r '$chan' 2>&1";
    die "Couldn't run $tzap" unless defined $tzap_pid;
    $SIG{TERM} = \&cleanup_and_exit;
    $SIG{HUP} = sub { $restart=1; };
    $restart = 0;
    #my $giveup_counter = 13;
    my $giveup_counter = 20;
    while ($giveup_counter > 0) {
	my $line = <TZAP>;
	exit(1) unless defined $line;
	print STDERR $line;
	last if $line =~ m/^status 1f .+ ber 0000.+ unc 0000000.+ FE_HAS_LOCK/;
	$giveup_counter--;
    }
    if ($giveup_counter == 0) {
	print STDERR "Couldn't get a signal.\n";
	&cleanup_and_exit();
	exit(0);	# Safety bumper
    }
    my $monitor_pid = fork();
    if ($monitor_pid == 0) {
	$SIG{HUP} = \&terminate_tzap_fork;
	my $uncorrectable = 0;
	my $checktime = time;
	while (1) {
	    my $line = <TZAP>;
	    last unless defined $line;
	    if ($line !~ m/^status 1f .+ unc (.{8}) .+ FE_HAS_LOCK/) {
		print STDERR "Houston, we have a problem:\n$line";
		print STDERR "Bad DVB signal status, giving up.\n";
		last;
	    }
	    $uncorrectable += hex($1);
	    if ($uncorrectable > 10000) {
		print STDERR "Houston, we have a problem:\n$line";
		print STDERR "Too many uncorrectable errors in DVB signal, giving up.\n";
		last;
	    }
	    if (time > $checktime + 60) {
		$uncorrectable = 0;
		$checktime = time;
	    }
	}
	kill 'SIGTERM', $play_pid;
	exit(1);
    } else {
	open SCHAN, '>', "$PATHPREFIX/var/dvb-playing-channel"
	    or die "Couldn't write to $PATHPREFIX/var/dvb-playing-channel: $!";
	print SCHAN $chan,"\n";
	close SCHAN;
	#
	# Copy the video stream
	#
	open MPEG, '<', $dvr_device
	    or die "Couldn't open $dvr_device for reading: $!";
	while (!$restart) {
	    my $iobuf = "";
	    my $n = sysread(MPEG, $iobuf, $mpegbufsize);
	    if ($n) {
		syswrite(STDOUT, $iobuf, $n);
	    } else {
		&cleanup_and_exit();
		exit(1);	# Safety bumper
	    }
	}
	close MPEG;
	print STDERR "Closed the DVB capture...\n";
    }
    kill 'SIGHUP', $monitor_pid;
    unlink "$PATHPREFIX/var/dvb-playing-channel";
    sleep 10.1;
}

